function [Sx EuroPrice AmerPrice] = BAWPrice(S,K,r,q,v,T,PutCall)

% Barone-Adesi and Whaley quadratic approximation for American vanilla options
% Finds the American prices as the European prices + premium
% Premium is based on Sx, the critical stock price above or below
% which it becomes optimal to exercise the American options
% S = Spot price
% K = Strike price
% r = Risk free rate
% q = Dividend yield
% v = Volatility
% T = Maturity
% PutCall = 'C'all or 'P'ut

% by Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Anonymous functions for Black Scholes call and put
BSCall = @(s,K,r,q,v,T) s.*exp(-q.*T).*normcdf((log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T)) - K.*exp(-r.*T).*normcdf((log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T) - v.*sqrt(T));
BSPut  = @(s,K,r,q,v,T) K.*exp(-r.*T).*normcdf(-(log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T) + v.*sqrt(T)) - s.*exp(-q.*T).*normcdf(-(log(s./K) + (r-q+v.^2./2).*T)./v./sqrt(T));


% Quadratic approximation
switch PutCall
	case 'C'
		EuroPrice = BSCall(S,K,r,q,v,T);
		start = S;
		[Sx feval] = fminsearch(@(s) findSx(s,K,r,q,v,T,PutCall), start);
		d1 = (log(Sx/K) + (r-q+v^2/2))/v/sqrt(T);
		n = 2*(r-q)/v^2;
		k = 2*r/v^2/(1-exp(-r*T));
		q2 = (1-n+sqrt((n-1)^2+4*k))/2;
		A2 = Sx*(1-exp(-q*T)*normcdf(d1))/q2;
		if S<Sx
			AmerPrice = EuroPrice + A2*(S/Sx)^q2;
		else
			AmerPrice = S - K;
		end
	case 'P'
		EuroPrice = BSPut(S,K,r,q,v,T);
		start = S;
		[Sx feval] = fminsearch(@(s) findSx(s,K,r,q,v,T,PutCall), start);
		d1 = (log(Sx/K) + (r-q+v^2/2))/v/sqrt(T);
		n = 2*(r-q)/v^2;
		k = 2*r/v^2/(1-exp(-r*T));
		q1 = (1-n-sqrt((n-1)^2+4*k))/2;
		A1 = -Sx*(1-exp(-q*T)*normcdf(-d1))/q1;
		if S>Sx
			AmerPrice = EuroPrice + A1*(S/Sx)^q1;
		else
			AmerPrice = K - S;
		end
end

